#!/usr/bin/python
# -*- coding:utf-8 -*-
"""
    Entrance of Bosftp
"""

# set encoding
import sys
reload(sys) # Python2.5 初始化后会删除 sys.setdefaultencoding 这个方法，我们需要重新载入
sys.setdefaultencoding('utf-8')

import logging
from logging.handlers import RotatingFileHandler

# pyftpdlib
from pyftpdlib.servers import FTPServer
from pyftpdlib.handlers import FTPHandler

# bos
from baidubce.bce_client_configuration import BceClientConfiguration
from baidubce.auth.bce_credentials import BceCredentials

# bosftp
from bosftp_config import BosftpCfg
from bos_authorizer import BosAuthorizer
from bos_file_system import BosAbstractFileSystem

def set_logger():
    """
    set logger
    """
    #log related
    MAXLOGSIZE = 1024 * 1024 * 1024 #Bytes
    BACKUPCOUNT = 30
    FORMAT = \
        "%(asctime)s %(levelname)-8s[%(filename)s:%(lineno)d(%(funcName)s)] %(message)s"
    handler = RotatingFileHandler(BosftpCfg().log_file_name,
                mode='w',
                maxBytes=MAXLOGSIZE,
                backupCount=BACKUPCOUNT)
    formatter = logging.Formatter(FORMAT)
    handler.setFormatter(formatter)
    logger = logging.getLogger()
    logger.setLevel(BosftpCfg().log_level)
    logger.addHandler(handler)


def start(port):
    authorizer = BosAuthorizer()
    for login_user, login_password, permission in BosftpCfg().login_users:
        perm = ""
        if "R" in permission:
            perm = perm + "elr"
        if "W" in permission:
            perm = perm + "adfmwMT"
        print BosftpCfg().homedir
        authorizer.add_user(login_user, login_password, BosftpCfg().homedir, perm=perm)

    handler = FTPHandler
    handler.abstracted_fs = BosAbstractFileSystem
    handler.authorizer = authorizer
    handler.banner = "Welcome to BOS FTP Service"
    handler.permit_foreign_addresses = True

    if BosftpCfg().masquerade_address is not None:
        handler.masquerade_address = BosftpCfg().masquerade_address 
    server = FTPServer(("0.0.0.0", port), handler)	
    server.max_cons = BosftpCfg().max_connection_num
    set_logger()

    handler.passive_ports = BosftpCfg().passive_ports 

    print "running ftp server at port %d..." % (port)

    try:
        server.serve_forever()
    finally:
        server.close_all()


def main():
    listen_port = BosftpCfg().listen_port
    start(listen_port)

if __name__ == "__main__":
    main() 

