# -*- coding:utf-8 -*-
"""
    Test for Directory Operation
"""
import os
import unittest
import time
from socket import error as sock_error
from ftplib import error_temp
from ftplib import FTP
from ftplib import error_perm
import sys
sys.path.append("../bosftp")
import start_ftp_server 
from bosftp_config import BosftpCfg
from util import gen_file
from util import safe_remove_file
from util import random_string

HOST = "127.0.0.1"
USER = "xxx"
PASSWORD = "xxx"

class DirTest(unittest.TestCase):
    """
        A Test Class for testing directory operation
    """

    def setUp(self):
        self.ftp = FTP()
        self.ftp.connect(HOST, BosftpCfg().listen_port)
        self.ftp.login(USER, PASSWORD)

    def tearDown(self):
        self.ftp.quit()

    def test_mk_dir(self):
        print "test_mk_dir\n" 
        dir_name_list = [(random_string(255), 1), 
                         ('中文目录', 1), # 一层目录
                         ('中文目录测试!@@#)(*&^%$__＊&……&+-%w@=__测试这是目录!qwer1234', 1), 
                         ('/测试目录1层/测试目录2层', 2), # 2层目录
                         ('测目录1层/测试目录2层', 2),
                         ('/a/b/', 2),
                         ('a/b/c', 3),
                         ('a/b/c/d/', 4)]

        # clear
        for dir_name in dir_name_list:
            try:
                dir_name[0] = dir_name[0].rstrip("/")
                self.ftp.cwd(dir_name[0])
                self.ftp.rmd(dir_name[0])
            except:
                pass

        for dir_name in dir_name_list:
            try:
                self.ftp.mkd(dir_name[0])
                self.ftp.cwd(dir_name[0])
                for i in range(dir_name[1]):
                    self.ftp.cwd("..")
                self.ftp.rmd(dir_name[0])
            except (error_temp, error_perm, sock_error) as e:
                print e
                self.fail("create dir error")

    def test_cwd_dir(self):
        pass
        print "\n-------test_cwd_dir-----\n" 
        try:
            dir_name = "/" 
            data = []
            self.ftp.dir(data.append)
            for line in data:
                if line.startswith('drw'):
                    dir_name = line.split("\t")[-1]
                    self.ftp.cwd(dir_name)
                    data = []
                    self.ftp.dir(data.append)
                    self.ftp.cwd("..")
        except (error_temp, error_perm, sock_error) as e:
            print e.message
            self.fail("cwd dir error")

    def test_chdir_empty(self):
        print "test_chdr_empty"
        try:
            self.ftp.cwd("")
        except (error_temp, error_perm, sock_error):
            self.fail("cwd dir error")

    def test_list_dir(self):
        print "test_list_dir"
        test_cases = [unicode(time.time()), "中文目录", "%+_#$@&中文"]
        for dir_name in test_cases:
            try:
                self.ftp.rmd(dir_name)
            except:
                pass
        for test_dir_name in test_cases:
            try:
                self.ftp.mkd(test_dir_name)
                self.ftp.cwd(test_dir_name)
                file_1 = "file_1" + unicode(time.time())
                gen_file(file_1, 1024)
                self.ftp.storbinary("STOR %s" % file_1, open(file_1))
                file_2 = "file_2" + unicode(time.time())
                gen_file(file_2, 1024)
                self.ftp.storbinary("STOR %s" % file_2, open(file_2)) 
                tmp_dir = "tmp_dir" + unicode(time.time())
                self.ftp.mkd(tmp_dir)
                data = []
                self.ftp.dir(data.append)
                self.assertEqual(len(data), 3)
                data = []
                self.ftp.retrlines('MLSD', data.append)
                self.assertEqual(len(data), 3)
                self.ftp.cwd(tmp_dir)
                data = []
                self.ftp.retrlines('MLSD', data.append)
                self.assertEqual(len(data), 0)

                self.ftp.cwd("..")
                #clean
                self.ftp.rmd(tmp_dir)
                self.ftp.delete(file_1)
                self.ftp.delete(file_2)
                self.ftp.cwd("..")
                self.ftp.rmd(test_dir_name)
            except (error_temp, error_perm, sock_error) as e:
                print e
                self.fail("list dir error")
        
            safe_remove_file(file_1)
            safe_remove_file(file_2)

    def test_list_root_dir(self):
        print "test_list_root_dir"
        try:
            self.ftp.cwd('/')
            data = []
            self.ftp.retrlines('MLSD /', data.append)
        except (error_temp, error_perm, sock_error):
            self.fail("list dir error")

    def test_rm_dir(self):
        print "test_rm_dir"
        test_dir_name = unicode(time.time())
        try:
            self.ftp.mkd(test_dir_name)
            self.ftp.cwd(test_dir_name)
            self.ftp.cwd("..")
            self.ftp.rmd(test_dir_name)
        except (error_temp, error_perm, sock_error):
            self.fail("rm dir error")
        
        try:
            self.ftp.cwd(test_dir_name)
            self.fail("rmd dir error")
        except (error_temp, error_perm, sock_error):
            pass

    def test_rename_dir(self):
        print "test_rename_dir"
        test_dir_name = unicode(time.time())
        test_dir_name_to = test_dir_name + "_to"
        try:
            self.ftp.mkd(test_dir_name)
            self.ftp.rename(test_dir_name, test_dir_name_to)
            self.ftp.rmd(test_dir_name)
        except error_perm as e:
            self.assertEqual(e[0], "550 Directory renaming is not supported.")
        else:
            self.fail("check rename failed")

if __name__ == '__main__':
    #t = myThread("thread_id_1")
    #t.daemon = True
    #t.start()
    time.sleep(5)
    unittest.main()
