"""
    utils for unittest
"""
import os
import string
import threading
import random
import hashlib
import ConfigParser

from ftplib import FTP
from ftplib import error_temp
from ftplib import error_perm
from socket import error as sock_error

def random_string(n):
    """
    generate random_string
    """
    return ''.join(random.choice(string.ascii_lowercase) for i in range(n))


def gen_file(filename, file_size):
    """
    it will generate a file with given filename and file_size
    """
    if (os.path.isfile(filename)) and (file_size == os.path.getsize(filename)):
        return
    elif file_size >= 0 and file_size <= 10 * 1024 * 1024:
        f = open(filename, 'w')
        f.write(random_string(file_size))
        f.close()
    elif file_size > 10 * 1024 * 1024: #  > 10M file
        f = open(filename, 'w')
        f.write(random_string(1024))
        f.seek(file_size - 1024)
        f.write(random_string(1024))
        f.close()

def get_file_md5(filename, blocksize=2 ** 20):
    """
    caculate file md5
    """
    m = hashlib.md5()
    with open(filename, "rb") as f:
        while True:
            buf = f.read(blocksize)
            if not buf:
                break
            m.update(buf)
    return m.hexdigest()
#def get_file_md5(file_path):
#    md5 = hashlib.md5()
#
#    f = open(file_path)
#    for line in f:
#        md5.update(line)
#    f.close()
#    return md5.hexdigest()


def safe_remove_file(file_name):
    """
    remove file if it is regular file
    """
    if os.path.isfile(file_name):
        os.remove(file_name)


def storebinary(ftp, file_name, path):
    """
    STOR file to ftp server
    """
    try:
        ftp.storbinary("STOR %s" % path, open(file_name, 'rb'))
    except (error_temp, error_perm, sock_error) as e:
        print e
        return False
    return True


def retrbinary(ftp, path, file_name):
    """
    RETR(get) file from ftp server
    """
    try:
        ftp.retrbinary("RETR %s" % path, open(file_name, 'wb').write)
    except (error_temp, error_perm, sock_error) as e:
        print e
        return False
    return True


def delete_file(ftp, path):
    """
    DELETE file from ftp server
    """
    try:
        ftp.delete(path)
    except (error_temp, error_perm, sock_error) as e:
        print e
        return False
    return True


def get_size(ftp, path):
    """
    Get remote file size
    """
    try:
        size = ftp.size(path)
    except (error_temp, error_perm, sock_error) as e:
        print e
        return -1
    return size 

